﻿package com.nhe.video {
import flash.display.Sprite;
import flash.events.AsyncErrorEvent;
import flash.events.ErrorEvent;
import flash.events.IOErrorEvent;
import flash.events.SecurityErrorEvent;
import com.nhe.utils.EnterFrameBeacon
import flash.display.MovieClip;
import flash.events.Event;
import flash.events.NetStatusEvent;
import flash.events.EventDispatcher;
import flash.media.Sound;
import flash.media.Video;
import flash.net.NetConnection;
import flash.net.NetStream;
import flash.media.SoundTransform


/**
* Loads a video. Starts to load a video and dispatches events accordingly
*
* @since	Flash 9
* @author	Niko Helle
* @version	1.0.0
* @description	Loads video (flv) from given URL / path
* @usage Drag the movieclip from the library and scale it if necessary. If _autoSize is set to false, the video is scaled to instance size. 
* Set parameters in the properties panel or use ActionScript.
* <code>
* _videoLoader = _root.attachMovie("VideoLoader","f",1);
* _videoLoader.setSize(200,200);
* _videoLoader.load("file.flv");
* 
* </code>
* 
* Events:
* "readyToPlay": video is buffered, possibly resized and at frame 0. If autoPlay is true, play is started automatically
* "videoFullyLoaded": the whole file is loaded
* "playing": play started
* "stopped": video stopped
* "paused": video paused
* "fileNotFound": file not found
* "invalidTime": tried to play video from time that is not buffered. Event object has a variable maxTime that holds the max time to play 
* "loadProgress": dispatched while video is loading. Event object has variables bytesLoaded and bytesTotal
* "playProgress": dispatched when video playing. Event object has variables time (current time) and length (length of the video)
* "StreamFinished": dispatched when video has ended
* all events objects has a variable "netStream". It is reference to the NetStream object. 
*
* @see	
*/


public class VideoLoader extends Sprite {
	
	private var _videoWidth:Number;
	private var _videoHeight:Number; 
	
	private var _originalWidth:Number;
	private var _originalHeight:Number; 

	
	private var _video:Video;
	private var _playChecker:MovieClip;
	private var _loadChecker:MovieClip;
	private var _beacon:EnterFrameBeacon;

	private var _sound:SoundTransform;

	private var _loading:Boolean=false;
	private var _loaded:Boolean=false;
	private var _buffered:Boolean=false;
	private var _status:Number=0; // 1= ready, 2=playing, 3 = playing & ready, 4 = fullyloaded, 7 = playing & ready & fullyloaded
	private var _autoLoad:Boolean=true;
	private var _autoPlay:Boolean=true;
	private var _autoSize:Boolean=true;
	private var _autoRewind:Boolean=false;
	private var _mute:Boolean=false;
	private var _resized:Boolean=false;
	private var _hidden:Boolean=false;
	private var _forcedStop:Boolean=false;
	private var _initalized:Boolean=false;

	private var _path:String=null;
	private var _listener:Object;
	
	private var _bytesLoaded:Number = -1;
	private var _bytesTotal:Number = -1;
	
	private var _bufferTime:Number = 0.1;
	private var _volume:Number = 1;
	private var _length:Number = 0;
	
	private var _netConnection:NetConnection = null;
	private var _netStream:NetStream  = null;
	
	private var _data:Object;
	
	public static var LOAD_PROGRESS:String= "LOAD_PROGRESS";
	public static var FIRST_FRAME_READY:String= "FIRST_FRAME_READY";
	public static var READY_TO_PLAY:String= "READY_TO_PLAY";
	public static var PLAYING :String= "PLAYING ";
	public static var STREAM_FINISHED:String= "STREAM_FINISHED";
	public static var STOPPED:String= "STOPPED";
	public static var PAUSED:String= "PAUSED";
	public static var BUFFER_EMPTY:String= "BUFFER_EMPTY";
	public static var FILE_NOT_FOUND:String= "FILE_NOT_FOUND";
	public static var SEEK_COMPLETE:String= "SEEK_COMPLETE";
	public static var INVALID_TIME:String= "INVALID_TIME";
	public static var BUFFER_FULL:String= "BUFFER_FULL";
	public static var PLAY_PROGRESS:String= "PLAY_PROGRESS";
	public static var FULLY_LOADED:String= "FULLY_LOADED";
	public static var CONNECTION_FAILED:String= "CONNECT_FAILED";
	public static var ERROR:String= "ERROR";
	
	public function VideoLoader (videoWidth:uint=0,videoHeight:uint=0) {
		
		_originalWidth = _videoWidth = videoWidth>0 ? videoWidth : this.width;
		_originalHeight = _videoHeight = videoHeight>0 ? videoHeight : this.height;
		this.scaleX= this.scaleY = 1;

		_beacon = new EnterFrameBeacon();
		addChild(_beacon);
		if(_hidden) this.alpha = 0;
		else this.alpha = 1;
		_initalized = true;
		if(_autoLoad && _path != null) load(_path);
	}

	/**
	* Starts to load a video from given url / path
	* 
	* @param path URL / path of the video (flv)
	*/
	public function load(p:String,server:String=null) {
		
		if(!_initalized) return;
		
		//log("load path:"+p);
		
		_status = 0;
		_bytesLoaded = -1;
		_bytesTotal = -1;
		_path = p;
		_loading = true;
		_buffered = false;
		_loaded = false;
		_resized = false;
		_forcedStop = false;
		_length = 0;
		_netConnection = new NetConnection();
		_netConnection.client = this;
		_netConnection.addEventListener(NetStatusEvent.NET_STATUS, onNetStreamStatus)
        _netConnection.addEventListener(SecurityErrorEvent.SECURITY_ERROR, netConnection_SECURITY_ERROR);
        _netConnection.addEventListener(AsyncErrorEvent.ASYNC_ERROR, netConnection_ASYNC_ERROR);
        _netConnection.addEventListener(IOErrorEvent.IO_ERROR,netConnection_IO_ERROR);
        _netConnection.addEventListener(IOErrorEvent.NETWORK_ERROR,netConnection_IO_ERROR);
		_netConnection.connect(server);
		//if(server == null) connected();
		
	}
	
	public function connected() {
		
		if (_netStream) return;
		//log("#######connected:");
		_netStream = new NetStream(_netConnection);
		_netStream.bufferTime = _bufferTime;
		_netStream.checkPolicyFile = true;
		_netStream.addEventListener(NetStatusEvent.NET_STATUS, onNetStreamStatus);
		_netStream.client = this;
		//log("_netStream:"+_netStream);
		_sound = _netStream.soundTransform;
		//log("_sound:" + _sound);
		
		if(!_video){
			_video = new Video(_videoWidth,_videoHeight);
			addChild(_video);
		}
		
		_video.attachNetStream(_netStream)
		
		//log("_video:"+_video);
		mute = _mute;
		_netStream.play(_path);
		_sound.volume = 0;
		_netStream.soundTransform = _sound;
		
		
		_beacon.addEventListener(Event.ENTER_FRAME, checkVideoLoad);
		
		this.alpha = 0;
	}
	
	public function checkVideoLoad(e:Event):void {
		if(!_autoSize) _resized = true;
		if(_forcedStop) {
			if(_hidden) _video.clear();
			else this.alpha = 1;
		}
		if(_loaded && _buffered && _resized && _forcedStop) {
			_beacon.removeEventListener(Event.ENTER_FRAME, checkVideoLoad);
			return;
		}
		if(!_forcedStop && Number(_netStream.currentFPS >0))  {
			if(!_autoPlay) {
				_netStream.seek(0);
				_netStream.pause();
			}
			_forcedStop = true;

		}
		if(!_buffered && _netStream.bufferLength > _bufferTime) {
			_buffered = true;

		}

		if(!_resized && _video.width > 0) {
			_resized = true;
		}
		if(_buffered && _resized && _forcedStop) onNetStreamStatus(new NetStatusEvent(NetStatusEvent.NET_STATUS,false,false,{code:"Player.ready"}));
		if(_autoPlay && _buffered && _resized && _forcedStop) onNetStreamStatus(new NetStatusEvent(NetStatusEvent.NET_STATUS,false,false,{code:"Player.start"}));
		if(_loaded) return;
		if(bytesLoaded < 50) return;
		if(bytesTotal < 50) return;
		dispatchEvent(new Event(LOAD_PROGRESS));
		if(bytesLoaded < bytesTotal) return;
		_loaded = true;
		onNetStreamStatus(new NetStatusEvent(NetStatusEvent.NET_STATUS,false,false,{code:"Player.videoFullyLoaded"}));//{code:"Player.videoFullyLoaded"});
	}


	public function onMetaData(infoObject:Object):void {
			_length = infoObject.duration;
	}
	
	public function onXMPData (...rest):void {
			//nuttin
	}
	
	public function onPlayStatus (...rest):void {
			//nuttin
	}
	 
	
	public function onBWCheck(...rest):Number {
			return 0;
	}
	
	public function onBWDone(...rest):void {
			//nuttin
	}
	
	public function onCaption(...rest):void {
			//nuttin
	}

	public function onImageData(...rest):void {
			//nuttin
	}
	
	public function RtmpSampleAccess(...rest):void {
			//nuttin
	}
	
	public function onTextData(...rest):void {
			//nuttin
	}

	
	public function netConnection_SECURITY_ERROR(e:SecurityErrorEvent):void {
			//trace("netConnection_SECURITY_ERROR:" + e.text)
			dispatchEvent(new ErrorEvent(ErrorEvent.ERROR,false,false,"Video Security Error:"+e.text));
	}
	
	public function netConnection_ASYNC_ERROR(e:AsyncErrorEvent):void {
			//trace("netConnection_ASYNC_ERROR:" + e.text)
			dispatchEvent(new ErrorEvent(ErrorEvent.ERROR,false,false,"Video Async Error:"+e.text));
	}
	
	public function netConnection_IO_ERROR(e:IOErrorEvent):void {
			dispatchEvent(new ErrorEvent(ErrorEvent.ERROR,false,false,"Video IO Error:"+e.text));
			//trace("netConnection_IO_ERROR:"+e.text)
	}
	
	
	
	public function onNetStreamStatus(nse:NetStatusEvent):void {

		var e = {};
		e.target = this;
		e.type = "";
		var infoObject = nse.info
		switch (infoObject.code) {
			case 'NetStream.Play.Start' :
				e.type = FIRST_FRAME_READY; 
				//log(e.type);
				break;
			case 'Player.ready' :
				if(_status & 1) break;
				_status += 1;
				if(_hidden)_video.clear();
				e.type = READY_TO_PLAY; 
				

				
				
				if(_autoSize) {
					_videoWidth = _video.width = _video.videoWidth;
					_videoHeight = _video.height = _video.videoHeight

				}
				else {
					_video.width = _videoWidth;
					_video.height = _videoHeight;
				}
				
				if (isNaN(_originalWidth)) _originalWidth = _video.width;
				if (isNaN(_originalHeight)) _originalHeight = _video.height;
				
				//trace("sizes set _video.width:"+_video.width);
				
				break;	
			case 'Player.videoFullyLoaded' :
				if(_status & 4) break;
				_status += 4;
				e.type = FULLY_LOADED; 
				//log(e.type);				
				//if(!_autoPlay) playVideo(0,false);
				break;		
			case 'Player.start' :
				//log("Player.start _autoPlay:"+_autoPlay);
				if(_status & 2) break;
				_status += 2;
				e.type = PLAYING 
				//log(e.type);
				playVideo(0,true);
				break;	
			case 'NetStream.Play.Stop':
				stopVideo(_autoRewind);
				e.type = STREAM_FINISHED; 
				//log(e.type);
				break;	
			case 'Player.Stop':
				if(_status & 2) _status -= 2;
				else break;
				e.type = STOPPED; 
				//log(e.type);
				break;	
			case 'Player.paused':
				if(Boolean(_status & 2)) _status -= 2;
				else break;
				e.type = PAUSED; 
				//log(e.type);
				break;		
			case 'NetStream.Buffer.Empty':
				e.type = BUFFER_EMPTY 
				//log(e.type);
				break;
			case 'NetStream.Play.StreamNotFound':
				e.type = FILE_NOT_FOUND; 
				//log(e.type);
				break;
			case 'NetStream.Buffer.Flush':
				//e.type = "StreamFinished";
				//log(e.type);
				break;	
			case 'NetStream.Play.Notify' :
				e.type = SEEK_COMPLETE 
				//log(e.type);
				break;	
			case 'NetStream.Seek.InvalidTime':
				e.type = INVALID_TIME; 
				e.maxTime = Number(infoObject.message.details);
				e.length = length;
				//log(e.type);
				break;		
			case 'NetStream.Buffer.Full':
				_buffered = true;
				

				e.type = BUFFER_FULL
				//log(e.type);
				e.play = _autoPlay;

				break;
			
			case 'NetConnection.Connect.Success':
				//trace("CONN OK");
				connected();

				break;
			
			case 'NetConnection.Connect.Failed':
				e.type = CONNECTION_FAILED; 

			break;

				
		}
		
		//log("e.type:"+e.type+","+this+", "+infoObject.code);
		e.netStream = _netStream;
		if(e.type != "") dispatchEvent(new Event(e.type));


	}

	public function checkVideoProgress(e:Event){
		dispatchEvent(new Event(PLAY_PROGRESS));
	}
	
	/**
	* plays video from given time (seconds)
	* show() is also called, if the image is already loaded and "autoShow" is set to true
	* @param time Time where the play should start. If time exceeds the currently buffered time or the length of the video, event "invalidTime" is dispatched
	*/
	public function playVideo(i:Number=0,noEvent:Boolean=true){
		//log("***********playVideo playable:"+playable);
		if(!playable) {
			_autoPlay = true;
			return;
		}
		//log("playVideo _volume:"+_volume);
		
		_sound.volume = _volume;
		if(_mute) _sound.volume = 0;
		_netStream.soundTransform = _sound;
		if(i == 0) _netStream.resume()
		else _netStream.seek(i);
		
		if(noEvent === false) {if(!Boolean(_status & 2)) _status += 2;}
		else onNetStreamStatus(new NetStatusEvent(NetStatusEvent.NET_STATUS,false,false,{code:"Player.start"}));
		
		
		_beacon.addEventListener(Event.ENTER_FRAME, checkVideoProgress);
	}
	
	/**
	* pauses video 
	*/
	public function pause(){
		if(!playable) {
			_autoPlay = false;
			return;
		}
		_netStream.pause();
		_beacon.removeEventListener(Event.ENTER_FRAME, checkVideoProgress);
		onNetStreamStatus(new NetStatusEvent(NetStatusEvent.NET_STATUS,false,false,{code:"Player.paused"}));
		
	}
	
	
	public function seek(i:Number){
		_netStream.seek(i);
	}

	/**
	* stops video 
	*/
	public function stopVideo(rew:Boolean=false){
		if(!playable) {
			_autoPlay = false;
			return;
		}
		if(rew === false) rew = false;
		else rew = true;
		
		if(rew) {
			_netStream.seek(0);
			_netStream.pause();
		}
		//if(Boolean(_status & 2)) _status -= 2;
		_beacon.removeEventListener(Event.ENTER_FRAME, checkVideoProgress);
		//onNetStreamStatus( { code:"Player.Stop" } );
		onNetStreamStatus(new NetStatusEvent(NetStatusEvent.NET_STATUS,false,false,{code:"Player.Stop"}));
		
	}
	
	/**
	* Stops the video and clears the image currently displayed in the Video object.
	*/
	public function close(){
		stopVideo();
		_netStream.close();
		_video.clear();
		_video.attachNetStream(null);
		_netStream = null;
	}
	
	/**
	* cancel video loading
	*/
	public function cancel(){
		delete _loadChecker.onEnterFrame
		stopVideo();
		_status = 0;
		_netStream.close();

		
	}
	
	/**
	* returns true if loading is in progress
	* @return Boolean
	*/
	public function get loading():Boolean{
		return _loading;
	}

	/**
	* returns true if video is fully loaded
	* @return Boolean
	*/
	public function get loaded():Boolean{
		return _loaded;
	}
	
	/**
	* if "autoSize" is true, the video is resized to the original size of the video
	* @param value Boolean value: true / false.  Default true.
	*/
	[Inspectable(defaultValue=true, type=Boolean)]
	public function set autoSize(b:Boolean):void{
		_autoSize = b;
	}
	
	/**
	* returns current "autoSize" value
	* @return Boolean
	*/
	public function get autoSize():Boolean{
		return _autoSize;
	}

	/**
	* if "autoLoad" is true,loading starts automatically when "path" is set
	* loading is started, if "path" is already set and "autoLoad" is set to true
	* @param value Boolean value: true / false.  Default true.
	*/
	[Inspectable(defaultValue=true, type=Boolean)]
	public function set autoLoad(b:Boolean):void{
		_autoLoad = b;
		if(b && path != null) load(path);
	}
	
	/**
	* returns current "autoLoad" value
	* @return Boolean
	*/
	public function get autoLoad():Boolean{
		return _autoLoad;
	}
	
	/**
	* if "autoPlay" is true, the video is played when buffer length reaches bufferTime
	* @param value Boolean value: true / false.  Default true.
	*/
	[Inspectable(defaultValue=true, type=Boolean)]
	public function set autoPlay(b:Boolean):void{
		_autoPlay = b;
		if(b && _loaded) playVideo();
	}
	
	/**
	* returns current "autoPlay" value
	* @return Boolean
	*/
	public function get autoPlay():Boolean{
		return _autoPlay;
	}
	
	/**
	* if "autoRewind" is true, the video playhead is set to the beginning after the video is at end
	* @param value Boolean value: true / false.  Default false.
	*/
	[Inspectable(defaultValue=false, type=Boolean)]
	public function set autoRewind(b:Boolean):void{
		_autoRewind = b;
		
	}
	
	/**
	* returns current "autoRewind" value
	* @return Boolean
	*/
	public function get autoRewind():Boolean{
		return _autoRewind;
	}
	
	

	/**
	* URL / path of the video (flv)
	* loading is started automatically, if "autoLoad" is set to true
	* @param value String value
	*/
	
	[Inspectable(defaultValue="", type=String)]
	public function set path(s:String):void{
		if(s.indexOf(".") == -1) _path = null;
		else _path = s
		if(_autoLoad && _path != null) load(path);
	}
	
	/**
	* returns current "path"
	* @return String
	*/
	public function get path():String{
		return _path;
	}
	
	/**
	* set the length of the buffer before playing the video
	* if "autoPlay" is true, the video is played when buffer length reaches bufferTime
	* @param value Number value (seconds). Default is 0.1
	*/
	[Inspectable(defaultValue=0.1, type=Number)]
	public function set bufferTime(s:Number):void{
		_bufferTime = s;
	}
	
	/**
	* Sets size of the video. Sets "autoSize" to false also.
	* @param width Number value
	* @param height Number value
	*/
	public function setSize(w:Number,h:Number){
		_videoWidth = w;
		_videoHeight = h;
		_autoSize = false;
		_video.width = _videoWidth;
		_video.height = _videoHeight;
	}
	
	/**
	* returns width of the video object
	* @return Number
	*/
	public function get videoWidth():Number{
		return _videoWidth;
	}
	
	/**
	* returns height of the video object
	* @return Number
	*/
	public function get videoHeight():Number{
		return _videoHeight;
	}
	
	/**
	* returns the original width of  the video
	* @return Number
	*/
	public function get originalWidth():Number{
		return _originalWidth;
	}
	
	/**
	* returns the original height of  the video
	* @return Number
	*/
	public function get originalHeight():Number{
		return _originalHeight;
	}
	
	/**
	*  returns the width of the video
	* @return Number
	*/
	override public function get width():Number{
		if(!autoSize) return originalWidth;
		return videoWidth;
	}
	
	/**
	*  returns the height of the video
	* @return Number
	*/
	override public function get height():Number{
		if(!autoSize) return originalHeight;
		return videoHeight;
	}
	
	/**
	* returns current "bytesLoaded" value
	* @return Number
	*/
	public function get bytesLoaded():Number{
		if(!_loading && !_loaded) return 0;
		return _netStream.bytesLoaded;
	}
	
	/**
	* returns current "bytesTotal" value
	* @return Number
	*/
	public function get bytesTotal():Number{
		if(!_loading && !_loaded) return 0;
		return _netStream.bytesTotal;
	}
	
	/**
	* Sets volume of the video.
	* @param volume Number value (0-1). Default 1.
	*/
	[Inspectable(defaultValue=1, type=Number)]
	public function set volume(s:Number):void{
		_volume = s;
		_sound.volume = _volume;
		_netStream.soundTransform = _sound;
		//log("set volume:"+_volume);
		
	}
	
	/**
	* returns current "volume" value
	* @return Number
	*/
	public function get volume():Number{
		return _volume;
	}
	
	/**
	* Mutes the video / cancels the muting
	* @param value Boolean true / false
	*/
	public function set mute(b:Boolean):void{
		_mute = b;
		if(b)_sound.volume = 0;
		else _sound.volume = _volume;
		_netStream.soundTransform = _sound;
	}
	
	/**
	* returns current "mute" value
	* @return Boolean
	*/
	public function get mute():Boolean{
		return _mute;
	}
	
	/**
	* hides the video 
	* @param value Boolean true / false
	*/
	[Inspectable(defaultValue=false, type=Boolean)]
	public function set hidden(b:Boolean):void{
		_hidden = b;
		if(b) _video.clear();
		if(b) this.alpha = 0;
		else this.alpha = 1;

	}
	
	/**
	* returns current "hidden" value
	* @return Boolean
	*/
	public function get hidden():Boolean{
		return _hidden;
	}
	
	/**
	* returns the length of the video
	* @return Number
	*/
	public function get length():Number{
		return _length;
	}
	
	/**
	* returns true if video is ready to play
	* @return Boolean
	*/
	public function get playable():Boolean{
		return Boolean(_status & 1);
	}
	
	/**
	* returns true is video is playing
	* @return Boolean
	*/
	public function get isPlaying():Boolean{
		return Boolean(_status & 2);
	}
	
	public function get initialized():Boolean{
		return _initalized
	}
	
	public function get buffered():Boolean{
		return _buffered;
	}
	
	
	/**
	* set optional data object
	* @param value Object 
	*/
	public function set data(b:Object):void{
		_data = b;
		
	}
	
	/**
	* returns data object
	* @return Object
	*/
	public function get data():Object{
		return _data;
	}
	
	public function get video():Video { return _video; }
	
	
	public function log(s:String):void {
		trace("VideoLoader:"+s);
		
	}
	
	
		
}
}